@@ -52,6 +52,7 @@ setup_pipeline (const gchar * pipe_descr)
   fail_unless (pipeline != NULL, "Failed to create pipeline %s", pipe_descr);
   /* Newly returned object should be floating reffed */
   fail_unless (g_object_is_floating (pipeline));
+  g_assert_cmpuint (G_OBJECT (pipeline)->ref_count, ==, 1);
   return pipeline;
 }
 
@@ -130,6 +131,7 @@ GST_START_TEST (test_launch_lines)
   type = gst_element_factory_get_element_type (efac);
   fail_unless (type != 0);
   g_object_unref (efac);
+  g_object_unref (efac);
   fail_unless (gst_element_register (NULL, "1__dentity", GST_RANK_NONE, type));
 
   for (s = test_lines; *s != NULL; s++) {
@@ -467,6 +469,7 @@ run_delayed_test (const gchar * pipe_str, const gchar * peer,
     fail_if (peer_elem == NULL, "Could not retrieve peer %s", peer);
 
     sinkpad = gst_element_get_static_pad (peer_elem, "sink");
+    gst_object_unref (peer_elem);
     fail_if (sinkpad == NULL, "Peer element did not have a 'sink' pad");
 
     fail_unless (peerpad == sinkpad,
@@ -545,8 +548,8 @@ gst_parse_test_element_class_init (GstParseTestElementClass * klass)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&test_element_pad_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &test_element_pad_template);
 
   gst_element_class_set_metadata (gstelement_class,
       "Test element for parse launch tests", "Source",
