@@ -56,8 +56,6 @@
 #define GST_DISABLE_GST_DEBUG 1
 #define GST_DISABLE_LOADSAVE 1
 #define GST_DISABLE_PARSE 1
-#define GST_DISABLE_TRACE 1
-#define GST_DISABLE_ALLOC_TRACE 1
 #define GST_DISABLE_REGISTRY 1
 #define GST_DISABLE_PLUGIN 1
 #define GST_HAVE_GLIB_2_8 1
@@ -76,18 +74,9 @@
 /* wether or not the debugging subsystem is enabled */
 /* #undef GST_DISABLE_GST_DEBUG */
 
-/* DOES NOT WORK */
-/* #undef GST_DISABLE_LOADSAVE */
-
 /* DOES NOT WORK */
 /* #undef GST_DISABLE_PARSE */
 
-/* DOES NOT WORK */
-/* #undef GST_DISABLE_TRACE */
-
-/* DOES NOT WORK */
-/* #undef GST_DISABLE_ALLOC_TRACE */
-
 /* DOES NOT WORK */
 /* #undef GST_DISABLE_REGISTRY */
 
@@ -125,14 +114,6 @@
    refcounting */
 #define GST_HAVE_GLIB_2_8 1
 
-/***** Deal with XML stuff, we have to handle both loadsave and registry *****/
-
-#if (! (defined(GST_DISABLE_LOADSAVE) && defined(GST_DISABLE_REGISTRY)) )
-# include <libxml/parser.h>
-#else
-# define GST_DISABLE_LOADSAVE_REGISTRY
-#endif
-
 /**
  * GST_EXPORT:
  *
